package ch.psi.daq.test.queryrest.controller;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.junit.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.query.model.AggregationType;
import ch.psi.daq.query.model.PulseRangeQuery;
import ch.psi.daq.query.model.TimeRangeQuery;
import ch.psi.daq.test.queryrest.AbstractDaqRestTest;

/**
 * Tests the {@link DaqController} implementation.
 */
public class DaqRestControllerTest extends AbstractDaqRestTest {

   private static final List<String> DEFAULT_PROPERTIES = Lists.newArrayList("channel", "pulseId");

   @Test
   public void testPulseRangeQuery() throws Exception {
      PulseRangeQuery request = new PulseRangeQuery(
            100,
            101
            );
      request.setOrdering(Ordering.desc);
      request.setFields(Sets.newLinkedHashSet(DEFAULT_PROPERTIES));
      request.setNrOfBins(100);
      request.setAggregateChannels(false);
      request.setAggregationType(AggregationType.index);

      String content = mapper.writeValueAsString(request);

      this.mockMvc
            .perform(MockMvcRequestBuilders.post("/pulserange")
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"));
   }

   @Test
   public void testTimeRangeQuery() throws Exception {
      long startTime = new Date().getTime();
      long endTime = startTime + TimeUnit.SECONDS.toMillis(1);
      
      TimeRangeQuery request = new TimeRangeQuery(
            startTime,
            endTime,
            "test");
      request.setOrdering(Ordering.asc);
      request.setFields(Sets.newLinkedHashSet(DEFAULT_PROPERTIES));
      request.setNrOfBins(100);
      request.setAggregateChannels(false);
      request.setAggregationType(AggregationType.index);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders.post("/timerange")
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"));
   }

   @Test
   public void testMapper() throws JsonProcessingException {
      long startTime = new Date().getTime();
      long endTime = startTime + TimeUnit.SECONDS.toMillis(1);
      TimeRangeQuery request = new TimeRangeQuery(
            startTime,
            endTime,
            "test");
      request.setOrdering(Ordering.asc);
      request.setFields(Sets.newLinkedHashSet(DEFAULT_PROPERTIES));
      request.setNrOfBins(100);
      request.setAggregateChannels(false);
      request.setAggregationType(AggregationType.index);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);
   }
}
